@@ -25,7 +25,8 @@
 
 /* This is only a test program, so we don't need translations. Still keep the
  * infrastructure in place in case we suddenly decide we want to localize this
- * program. */
+ * program. Don't forget to reenable the call to bindtextdomain() if going back
+ * to real localization. */
 #define _(x) x
 #define N_(x) x
 
@@ -48,7 +49,7 @@ append_directory_path (GMenuTreeDirectory *directory,
 {
   GMenuTreeDirectory *parent;
 
-  parent = gmenu_tree_item_get_parent (GMENU_TREE_ITEM (directory));
+  parent = gmenu_tree_directory_get_parent (directory);
 
   if (!parent)
     {
@@ -104,8 +105,7 @@ print_entry (GMenuTreeEntry *entry,
 static void
 print_directory (GMenuTreeDirectory *directory)
 {
-  GSList     *items;
-  GSList     *tmp;
+  GMenuTreeIter *iter;
   const char *path;
   char       *freeme;
 
@@ -115,34 +115,42 @@ print_directory (GMenuTreeDirectory *directory)
   else
     path = freeme + 1;
 
-  items = gmenu_tree_directory_get_contents (directory);
+  iter = gmenu_tree_directory_iter (directory);
 
-  tmp = items;
-  while (tmp != NULL)
+  while (TRUE)
     {
-      GMenuTreeItem *item = tmp->data;
+      gpointer item;
 
-      switch (gmenu_tree_item_get_type (item))
+      switch (gmenu_tree_iter_next (iter))
 	{
+	case GMENU_TREE_ITEM_INVALID:
+	  goto done;
+
 	case GMENU_TREE_ITEM_ENTRY:
-	  print_entry (GMENU_TREE_ENTRY (item), path);
+	  item = gmenu_tree_iter_get_entry (iter);
+	  print_entry ((GMenuTreeEntry*)item, path);
 	  break;
 
 	case GMENU_TREE_ITEM_DIRECTORY:
-	  print_directory (GMENU_TREE_DIRECTORY (item));
+	  item = gmenu_tree_iter_get_directory (iter);
+	  print_directory ((GMenuTreeDirectory*)item);
 	  break;
 
 	case GMENU_TREE_ITEM_HEADER:
 	case GMENU_TREE_ITEM_SEPARATOR:
+	  item = NULL;
 	  break;
 
 	case GMENU_TREE_ITEM_ALIAS:
 	  {
-	    GMenuTreeItem *aliased_item;
+	    item = gmenu_tree_iter_get_alias (iter);
 
-	    aliased_item = gmenu_tree_alias_get_item (GMENU_TREE_ALIAS (item));
-	    if (gmenu_tree_item_get_type (aliased_item) == GMENU_TREE_ITEM_ENTRY)
-	      print_entry (GMENU_TREE_ENTRY (aliased_item), path);
+	    if (gmenu_tree_alias_get_aliased_item_type (item) == GMENU_TREE_ITEM_ENTRY)
+	      {
+		GMenuTreeEntry *entry = gmenu_tree_alias_get_aliased_entry (item);
+		print_entry (entry, path);
+		gmenu_tree_item_unref (entry);
+	      }
 	  }
 	  break;
 
@@ -151,12 +159,13 @@ print_directory (GMenuTreeDirectory *directory)
 	  break;
 	}
 
-      gmenu_tree_item_unref (tmp->data);
-
-      tmp = tmp->next;
+      gmenu_tree_item_unref (item);
+      continue;
+    done:
+      break;
     }
 
-  g_slist_free (items);
+  gmenu_tree_iter_unref (iter);
 
   g_free (freeme);
 }
@@ -165,9 +174,17 @@ static void
 handle_tree_changed (GMenuTree *tree)
 {
   GMenuTreeDirectory *root;
+  GError *error = NULL;
 
   g_print (_("\n\n\n==== Menu changed, reloading ====\n\n\n"));
 
+  if (!gmenu_tree_load_sync (tree, &error))
+    {
+      g_printerr ("Failed to load tree: %s\n", error->message);
+      g_clear_error (&error);
+      return;
+    }
+
   root = gmenu_tree_get_root_directory (tree);
   if (root == NULL)
     {
@@ -186,10 +203,16 @@ main (int argc, char **argv)
   GMenuTree          *tree;
   GMenuTreeDirectory *root;
   GMenuTreeFlags      flags;
+  GError             *error = NULL;
 
+  g_type_init ();
+
+#if 0
+  /* See comment when defining _() at the top of this file. */
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
+#endif
 
   options_context = g_option_context_new (_("- test GNOME's implementation of the Desktop Menu Specification"));
   g_option_context_add_main_entries (options_context, options, GETTEXT_PACKAGE);
@@ -202,9 +225,17 @@ main (int argc, char **argv)
   if (include_nodisplay)
     flags |= GMENU_TREE_FLAGS_INCLUDE_NODISPLAY;
 
-  tree = gmenu_tree_lookup (menu_file ? menu_file : "applications.menu", flags);
+  tree = gmenu_tree_new (menu_file ? menu_file : "applications.menu", flags);
   g_assert (tree != NULL);
 
+  if (!gmenu_tree_load_sync (tree, &error))
+    {
+      g_printerr ("Failed to load tree: %s\n", error->message);
+      return 1;
+    }
+
+  g_print ("Loaded menu from %s\n", gmenu_tree_get_canonical_menu_path (tree));
+
   root = gmenu_tree_get_root_directory (tree);
   if (root != NULL)
     {
@@ -220,21 +251,14 @@ main (int argc, char **argv)
     {
       GMainLoop *main_loop;
 
-      gmenu_tree_add_monitor (tree,
-			      (GMenuTreeChangedFunc) handle_tree_changed,
-			      NULL);
+      g_signal_connect (tree, "changed", G_CALLBACK (handle_tree_changed), NULL);
 
       main_loop = g_main_loop_new (NULL, FALSE);
       g_main_loop_run (main_loop);
       g_main_loop_unref (main_loop);
-
-      gmenu_tree_remove_monitor (tree,
-				 (GMenuTreeChangedFunc) handle_tree_changed,
-				 NULL);
-
     }
 
-  gmenu_tree_unref (tree);
+  g_object_unref (tree);
 
   return 0;
 }
